--[==============================[
  =         Camera Class         =
--]==============================]


require "util.class"
require "util.vector2"


class("Camera",
	function (self, v_pos, rotation, zoom, v_offset)
		self.pos = v_pos or Vector2(0, 0) -- position relative to the origin
		self.rot = rotation or 0
		self.scale = zoom or 1
		self.scale_max = TUNING.CAMERA.ZOOM_MAX
		self.scale_min = TUNING.CAMERA.ZOOM_MIN
		-- self.offset = v_offset or Vector2(0, 0) -- additional offset away from the center.
	end
)


-- METAMETHODS --
Camera.__tostring = function (self)
	return string.format("Pos: %s - Zoom: %2.2f - Rot: %2.2f", tostring(self.pos), self.scale, self.rot)
end


-- POSITION METHODS --
Camera.moveTo = function (self, v_pos)
	self.pos = v_pos
	return self
end
Camera.move = function (self, v_move)
	self.pos = self.pos + v_move
	return self
end


-- ROTATION METHODS --
Camera.rotateTo = function (self, rads)
	self.rot = rads
	return self
end
Camera.rotate = function (self, rads)
	self.rot = self.rot + rads
	return self
end


-- ZOOM METHODS --
Camera.zoomTo = function (self, zoom, force)
	if force then
		self.scale = zoom
	else
		self.scale = math.clamp(zoom, self.scale_min, self.scale_max)
	end
	return self
end
Camera.zoom = function (self, mul, force)
	return self:zoomTo(self.scale * mul, force)
end


-- CANVAS METHODS --
Camera.attach = function (self)
	local cx, cy = love.graphics.getWidth()/(2*self.scale), love.graphics.getHeight()/(2*self.scale)
	love.graphics.push()
	love.graphics.scale(self.scale)
	love.graphics.translate(cx, cy)
	love.graphics.rotate(self.rot)
	love.graphics.translate(-self.pos.x, -self.pos.y)
	return self
end
Camera.detatch = function (self)
	love.graphics.pop()
	return self
end
Camera.draw = function (self, func)
	self:attach()
	func()
	self:detach()
end


-- COORDINATE METHODS --
Camera.cameraCoords = function(self, x, y)
	local w,h = love.graphics.getWidth(), love.graphics.getHeight()
	local c,s = math.cos(self.rot), math.sin(self.rot)
	x,y = x - self.pos.x, y - self.pos.y
	x,y = c*x - s*y, s*x + c*y
	return Vector2(x*self.scale + w/2, y*self.scale + h/2)
end
Camera.worldCoords = function(self, x, y)
	local w,h = love.graphics.getWidth(), love.graphics.getHeight()
	local c,s = math.cos(-self.rot), math.sin(-self.rot)
	x,y = (x - w/2) / self.scale, (y - h/2) / self.scale
	x,y = c*x - s*y, s*x + c*y
	return Vector2(x+self.pos.x, y+self.pos.y)
end
Camera.mousePos = function(self)
	return self:worldCoords(love.mouse.getPosition())
end





