--[=============================================[
  =         Simple Class Implimentation         =
--]=============================================]


function class (g_name, parent, constructor)
	local _class = {}
	if(g_name ~= nil and type(g_name) == "string") then
		-- class is automatically created globally (for strict.lua)
		rawset(_G, g_name, _class)
	end

	-- handle input args
	if not constructor and type(parent) == 'function' then
		-- constructor function was the first argument
		constructor = parent
		parent = nil
	elseif type(parent) == 'table' then
		-- class is a simple copy of the parent class, copy
		for i,v in pairs(parent) do
			_class[i] = v
		end
		_class.super = parent
	end

	_class.__index = _class

	_class._constructor = constructor
	_class.is_a = function (self, parent_class) -- called by <classname>:is_a (<parent_class>)
		local m = getmetatable(self)
		-- loop over parent classes
		while m do
			if m == parent_class then return true end
			m = m.super
		end
		return false
	end

	-- use a metatable to add a constructor
	local mt = {}

	-- this allows the constructor to be called by <classname>(<args>)
	mt.__call = function (class_tbl, ...)
		local obj = {}
		setmetatable(obj, _class)
		if _class._constructor then
			_class._constructor(obj,...)
		end
		return obj
	end

	setmetatable(_class, mt)

	return _class
end



--[==[
	Class usage:

	-- Case 1: only a constructor, no parent
	Class("Person", function (self, name)
			self._name = name
			self.getName = function (self)
				return self._name
			end
		end)

	local Bob = Person("bob")

	print("1: " .. tostring(Bob:is_a(Person))) 	-- returns true
	print("2: " .. Bob:getName()) 		-- returns "bob"

	-- case 2: extending an existing class
	Class("Student", Person, function (self, name, grade)
			self.super(self, name) -- this is necessary to inherit values/functions from the Parent class
			self._grade = grade
			self.getGrade = function (self)
				return self._grade
			end
		end)

	print("3: " .. tostring(Bob:is_a(Student))) 	-- returns false; Bob is only a Person, not a Student.

	local Bill = Student("bill", 5)

	print("4: " .. Bill:getName())		-- returns "bill"; Student inherited the function from Person
	print("5: " .. tostring(Bill:is_a(Student))) 	-- returns true
	print("6: " .. tostring(Bill:is_a(Person))) 	-- returns true; Since Bill is a Student, he is also a Person

	--[[OUTPUT:
		1: true
		2: bob
		3: false
		4: bill
		5: true
		6: true
	]]

]==]
