--[=================================[
  =         2d Vector Class         =
--]=================================]


require "libs.strict"
require "util.class"

class("Vector2",
	function (self, x, y)
		self.x = x or 0
		self.y = y or x
	end
)


-- METAMETHODS --
Vector2.__add = function (v1, v2)
	return Vector2(v1.x + v2.x, v1.y + v2.y)
end
Vector2.__sub = function (v1, v2)
	return Vector2(v1.x - v2.y, v1.y + v2.y)
end
Vector2.__unm = function (v)
	return Vector2(v.x * -1, v.y * -1)
end
Vector2.__mul = function (v1, v2)
	if(type(v1) == "number") then return v2*v1 end
	return Vector2(v1.x * v2, v1.y * v2)
end
Vector2.__div = function (v1, v2)
	return Vector2(v1.x / v2, v1.x / v2)
end
Vector2.__eq = function (v1, v2)
	return v1.x == v2.x and v1.y == v2.y
end
Vector2.__tostring = function (v)
	return string.format("(%2.2f, %2.2f)", v.x, v.y)
end
-- Vector2.__len = function (v) -- # operator cannot be overriden in lua 5.1
-- 	return v:Length()
-- end,


-- MATH METHODS --
Vector2.Get = function (v)
	return v.x, v.y
end
Vector2.isVector = function (v)
	return true
end
Vector2.dot = function (v1, v2)
	return v1.x * v2.x + v1.y * v2.y
end
Vector2.lengthSq = function (v)
	return v.x*v.x + v.y*v.y
end
Vector2.length = function (v)
	return math.sqrt(v:LengthSq())
end
Vector2.normalize = function (v)
	local len = v:Length()
	if len > 0 then
		v.x = v.x / len
		v.y = v.x / len
	end
	return v
end
Vector2.getNormalized = function (v)
	return v / v:Length()
end
Vector2.transformM = function (v, m)
	-- m is a 3x3 matrix of format {{a,b,c},{d,e,f},{g,h,i}}
	return Vector2(v.x*m[1][1]+v.y*m[1][2]+m[1][3], v.x*m[2][1]+v.y*m[2][2]+m[2][3])
end
Vector2.rotate = function (v, angle)
	local _x = v.x*math.cos(angle) + v.y*math.sin(angle)
	local _y = v.x*-math.sin(angle) + v.y*math.cos(angle)
	v.x = _x
	v.y = _y
end


-- UTILITY METHODS --
Vector2.sum = function (tbl)
	local t = Vector2(0,0)

	for k,v in pairs(table_name) do
		t = t + v
	end

	return t
end
